//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Actor(id)
{
	this._actorStatMaps = {};
	this._variableMap = {};
	
	this._inventory = rp.Inventory.create();
	this._inventory.setParentActor(this);
	
	this._registeredPanelList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

Actor.create = function(id)
{
	var actor = new Actor();
	
	actor._id = id;
	
	return actor;
}

Actor.createFromSave = function(data)
{
	var actor = new Actor();
	
	actor._id = data.id;
	
	actor._name = data.name;
	actor._title = data.title;
	actor._portraitImageId = data.portraitImageId;

	actor._variableMap = data.variableMap;
	
	for (var key in data.actorStatSaveMap)
	{
		var actorStatTemplateId = data.actorStatSaveMap[key].actorStatTemplateId;
		if (rpModule.getActorStatTemplate(actorStatTemplateId) == null)
		{
			rpEngine.logger.log('Actor.readFromSave -- Cannot load actor stat, actor stat template "%s" does not exist.', [actorStatTemplateId], rp.Logger.LEVEL_DEBUG, rp.Logger.SUBJECT_GAME);
			continue;
		}

		actor._addActorStat(actorStatTemplateId)
		actor.getStat(actorStatTemplateId).readFromSave(data.actorStatSaveMap[key]);
	}
	
	actor._inventory = new rp.Inventory.createFromSave(data.inventorySave, actor);
	
	return actor;
}

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Actor.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_id : '',
	
	_name : '',
	_title : '',
	_portraitImageId : '',
	
	_actorStatMaps : null, //ActorStatCategory.id => ActorStat.id => ActorStat
	_variableMap : null,
	
	_inventory : null,
	
	_registeredPanelList : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	_addActorStat : function(actorStatTemplateId)
	{
		//Chargement du template de la stat
		
		var actorStatTemplate = rpModule.getActorStatTemplate(actorStatTemplateId);
		if (actorStatTemplate == null)
		{
			rpEngine.logger.log('Actor._addActorStat -- Stat template "%s" does not exist.', [actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return null;
		}
		
		//Si la catégorie n'existe pas dans l'objet, on l'ajoute
		
		var actorStatCategory = actorStatTemplate.getActorStatCategory();
		if (actorStatCategory == null)
		{
			rpEngine.logger.log('Actor._addActorStat -- Stat template "%s" has no category.', [actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return null;
		}
		var actorStatCategoryId = actorStatCategory.getId();
		
		if (this._actorStatMaps[actorStatCategoryId] == null)
		{
			this._actorStatMaps[actorStatCategoryId] = {};
		}
		
		//Création de la stat
		
		var actorStat = rp.ActorStat.create(this, actorStatTemplate);
		this._actorStatMaps[actorStatCategoryId][actorStatTemplateId] = actorStat;
		
		this.updateActorStatSourceActorEffects(actorStat);	

		return actorStatTemplate;
	},
	
	registerPanel : function(guiPanel)
	{
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			if (this._registeredPanelList[i] === guiPanel)
			{
				return;
			}
		}

		this._registeredPanelList.push(guiPanel);
	},
	
	unregisterPanel : function(guiPanel)
	{
		for (var i = this._registeredPanelList.length - 1; i >= 0; i--)
		{
			if (this._registeredPanelList[i] === guiPanel)
			{
				this._registeredPanelList.splice(i, 1);
			}
		}
	},
	
	updateActorStatSourceActorEffects : function(actorStat)
	{
		var sourceActorEffectList = actorStat.getSourceActorEffectList();
		sourceActorEffectList.length = 0;

		for (var categoryKey in this._actorStatMaps)
		{
			for (var statKey in this._actorStatMaps[categoryKey])
			{
				var actorEffectList = this._actorStatMaps[categoryKey][statKey].getActorStatTemplate().getActorEffectList();
				for (var i = 0; i < actorEffectList.length; i++)
				{
					var actorEffect = actorEffectList[i];
					if (actorEffect.getTargetActorStatTemplate() === actorStat.getActorStatTemplate())
					{
						sourceActorEffectList.push(actorEffect);
					}
				}
			}
		}
		
		var inventoryItemLists = this._inventory.getItemLists();
		for (var categoryKey in this._inventory.getItemLists())
		{
			var inventoryItemList = inventoryItemLists[categoryKey];
			for (var i = 0; i < inventoryItemList.length; i++)
			{
				var actorEffectList = inventoryItemList[i].getInventoryItemTemplate().getEquipActorEffectList();
				for (var j = 0; j < actorEffectList.length; j++)
				{
					var actorEffect = actorEffectList[j];
					if (actorEffect.getTargetActorStatTemplate() === actorStat.getActorStatTemplate())
					{
						sourceActorEffectList.push(actorEffect);
					}
				}
			}
		}
		
		actorStat.updateAllValuesFinal();
	},

	updateActorStatTargetActorEffectsFromActorStat : function(actorStat)
	{
		var actorEffectList = actorStat.getActorStatTemplate().getActorEffectList();
		for (var i = 0; i < actorEffectList.length; i++)
		{
			var actorEffect = actorEffectList[i];
			var targetActorStat = this.getStat(actorEffect.getTargetActorStatTemplate().getId());

			this.updateActorStatSourceActorEffects(targetActorStat);
		}
	},

	updateActorStatTargetActorEffectsFromInventoryItem : function(inventoryItem)
	{
		var actorEffectList = inventoryItem.getInventoryItemTemplate().getEquipActorEffectList();
		for (var i = 0; i < actorEffectList.length; i++)
		{
			var actorEffect = actorEffectList[i];
			var targetActorStat = this.getStat(actorEffect.getTargetActorStatTemplate().getId());

			this.updateActorStatSourceActorEffects(targetActorStat);
		}
	},

	updateRegisteredPanelListStat : function(actorStat)
	{
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			this._registeredPanelList[i].updateActorStat(actorStat);
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	_getActorStatParentCategoryId : function(actorStatTemplateId)
	{
		for (var actorStatCategoryId in this._actorStatMaps)
		{
			if (this._actorStatMaps[actorStatCategoryId][actorStatTemplateId] != null)
			{
				return actorStatCategoryId;
			}
		}
		
		return null;
	},
	
	getId : function()
	{
		return this._id;
	},
	setId : function(value)
	{
		this._id = value;
	},
	
	getInventory : function()
	{
		return this._inventory;
	},
	
	getName : function()
	{
		return this._name;
	},
	setName : function(value)
	{
		this._name = value;
		
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			this._registeredPanelList[i].updateInfo();
		}
	},
	
	getPortraitImageId : function()
	{
		return this._portraitImageId;
	},
	setPortraitImageId : function(value)
	{
		this._portraitImageId = value;
		
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			this._registeredPanelList[i].updateImage();
		}
	},

	getStat : function(actorStatTemplateId)
	{
		//Vérification si la stat existe déjà
		
		var actorStatCategoryId = this._getActorStatParentCategoryId(actorStatTemplateId);
		
		//Si la stat est introuvable, on essaie de la créer avec sa valeur par défaut
		
		if (actorStatCategoryId == null)
		{
			var actorStat = this._addActorStat(actorStatTemplateId);
			if (actorStat == null)
			{
				rpEngine.logger.log('Actor.getStat -- Error while creating stat "%s".', [actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				return null;
			}
			
			actorStatCategoryId = actorStat.getActorStatCategory().getId();
		}
		
		return this._actorStatMaps[actorStatCategoryId][actorStatTemplateId];
	},
	
	getStatMaps : function()
	{
		return this._actorStatMaps;
	},
	
	getStatValueCurrentBase : function(statTemplateId)
	{
		var actorStat = this.getStat(statTemplateId);
		if (actorStat == null)
		{
			rpEngine.logger.log('Actor.getStatValueCurrentBase -- Actor "%s" has no stat "%s".', [this._id, actorStatTemplateId], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_GAME);
			return 0.0;
		}
		
		return actorStat.getValueCurrentBase();
	},
	setStatValueCurrentBase : function(actorStatTemplateId, value)
	{
		var actorStat = this.getStat(actorStatTemplateId);
		if (actorStat == null)
		{
			rpEngine.logger.log('Actor.setStatValueCurrentBase -- Actor "%s" has no stat "%s".', [this._id, actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		actorStat.setValueCurrentBase(value);
		
		this.updateRegisteredPanelListStat(actorStat);
	},

	getTitle : function()
	{
		return this._title;
	},
	setTitle : function(value)
	{
		this._title = value;
		
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			this._registeredPanelList[i].updateInfo();
		}
	},
	
	getVariableMap : function()
	{
		return this._variableMap;
	}
	
	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Actor;

//------------------------------------------------------------------------------------------------------------