//------------------------------------------------------------------------------------------------------------
//CLASS DEFINITION
//------------------------------------------------------------------------------------------------------------

function Inventory()
{
	this._inventoryItemLists = {};
	
	this._registeredPanelList = [];
}

//------------------------------------------------------------------------------------------------------------
//CONSTANTS
//------------------------------------------------------------------------------------------------------------

Inventory.OPTION_IS_EQUIPPED = 'isEquipped';
Inventory.OPTION_FORCE_EQUIP = 'forceEquip';

Inventory.OPTION_EQUIPPED_ITEMS_ONLY = 'equippedItemsOnly';
Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY = 'unequippedItemsOnly';

//------------------------------------------------------------------------------------------------------------
//STATIC FUNCTIONS
//------------------------------------------------------------------------------------------------------------

Inventory.create = function()
{
	var inventory = new Inventory();
	
	return inventory;
}

Inventory.createFromSave = function(data, parentActor)
{
	var inventory = new Inventory();
	
	inventory.setParentActor(parentActor);
	
	for (var key in data.inventoryItemSaveList)
	{
		var inventoryItemTemplateId = data.inventoryItemSaveList[key].inventoryItemTemplateId;
		if (rpModule.getInventoryItemTemplate(inventoryItemTemplateId) == null)
		{
			rpEngine.logger.log('Inventory.createFromSave -- Cannot load inventory item, inventory item template "%s" does not exist.', [actorStatTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			continue;
		}

		inventory.addItem(inventoryItemTemplateId, data.inventoryItemSaveList[key].itemCountUnequipped, {});
		inventory.addItem(inventoryItemTemplateId, data.inventoryItemSaveList[key].itemCountEquipped, {'isEquipped' : true});
	}
	
	return inventory;
}

//------------------------------------------------------------------------------------------------------------
//PROTOTYPE
//------------------------------------------------------------------------------------------------------------

Inventory.prototype =
{
	//------------------------------------------------------------------------------------------------------------
	//VARIABLES
	//------------------------------------------------------------------------------------------------------------

	_parentActor : null,
	//_parentLocation : null,
	
	_inventoryItemLists : null, //InventoryItemCategory.id => InventoryItem

	_registeredPanelList : null,

	//------------------------------------------------------------------------------------------------------------
	//INITIALIZERS - SERIALIZERS
	//------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	//GENERAL FUNCTIONS
	//------------------------------------------------------------------------------------------------------------

	addItem : function(inventoryItemTemplateId, itemCount, options)
	{
		options[Inventory.OPTION_IS_EQUIPPED] = ((options[Inventory.OPTION_IS_EQUIPPED] != null) ? options[Inventory.OPTION_IS_EQUIPPED] : false);

		var inventoryItem = this.getItem(inventoryItemTemplateId);
		if (inventoryItem == null)
		{
			//Chargement du template de l'item
			
			var inventoryItemTemplate = rpModule.getInventoryItemTemplate(inventoryItemTemplateId);
			if (inventoryItemTemplate == null)
			{
				rpEngine.logger.log('Inventory.addItem -- Item template "%s" does not exist.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				return null;
			}
			
			//Si la catégorie n'existe pas dans l'objet, on l'ajoute
			
			var inventoryItemCategory = inventoryItemTemplate.getInventoryItemCategory();
			if (inventoryItemCategory == null)
			{
				rpEngine.logger.log('Inventory.addItem -- Item template "%s" has no category.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
				return null;
			}
			
			var inventoryItemCategoryId = inventoryItemCategory.getId();
			if (this._inventoryItemLists[inventoryItemCategoryId] == null)
			{
				this._inventoryItemLists[inventoryItemCategoryId] = [];
			}
			
			//Création de l'item

			inventoryItem = rp.InventoryItem.create(this, inventoryItemTemplate, itemCount, 0);	
			this._inventoryItemLists[inventoryItemCategoryId].push(inventoryItem);
		}
		else
		{
			var newItemCount = inventoryItem.getItemCountUnequipped() + itemCount;
			inventoryItem.setItemCountUnequipped(newItemCount);
		}
		
		if (options[Inventory.OPTION_IS_EQUIPPED] == true)
		{
			this.equipItem(inventoryItemTemplateId, itemCount, options);
		}
		
		this._parentActor.updateActorStatTargetActorEffectsFromInventoryItem(inventoryItem);

		this.updateRegisteredPanelListItem(inventoryItem);
	},
	
	equipItem : function(inventoryItemTemplateId, itemCount, options)
	{
		options[Inventory.OPTION_FORCE_EQUIP] = ((options[Inventory.OPTION_FORCE_EQUIP] != null) ? options[Inventory.OPTION_FORCE_EQUIP] : false);

		var inventoryItemTemplate = rpModule.getInventoryItemTemplate(inventoryItemTemplateId);
		if (inventoryItemTemplate == null)
		{
			rpEngine.logger.log('Inventory.equipItem -- Inventory item template "%s" does not exist.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		if (inventoryItemTemplate.getIsEquippable() == false)
		{
			rpEngine.logger.log('Inventory.equipItem -- Inventory item template "%s" is not equippable.', [inventoryItemTemplateId], rp.Logger.LEVEL_ERROR, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var inventoryItem = this.getItem(inventoryItemTemplateId);
		if (inventoryItem == null)
		{
			return;
		}
		
		if (inventoryItem.getItemCountUnequipped() == 0)
		{
			return;
		}
		
		var itemCountToChange = Math.min(itemCount, inventoryItem.getItemCountUnequipped());
		var newItemCountUnequipped = inventoryItem.getItemCountUnequipped() - itemCountToChange;
		var newItemCountEquipped = inventoryItem.getItemCountEquipped() + itemCountToChange;
		
		inventoryItem.setItemCountUnequipped(newItemCountUnequipped);
		inventoryItem.setItemCountEquipped(newItemCountEquipped);

		this._parentActor.updateActorStatTargetActorEffectsFromInventoryItem(inventoryItem);

		this.updateRegisteredPanelListItem(inventoryItem);
	},
	
	removeItem : function(inventoryItemTemplateId, itemCount, options)
	{
		options[Inventory.OPTION_EQUIPPED_ITEMS_ONLY] = ((options[Inventory.OPTION_EQUIPPED_ITEMS_ONLY] != null) ? options[Inventory.OPTION_EQUIPPED_ITEMS_ONLY] : false);
		options[Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY] = ((options[Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY] != null) ? options[Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY] : false);

		if (itemCount <= 0)
		{
			rpEngine.logger.log('Inventory.removeItem -- "itemCount" is lower than or equal to 0.', [inventoryItemTemplateId], rp.Logger.LEVEL_WARNING, rp.Logger.SUBJECT_GAME);
			return;
		}
		
		var inventoryItem = this.getItem(inventoryItemTemplateId);
		if (inventoryItem == null)
		{
			return;
		}
		
		var itemCountToRemove = itemCount;
		
		if (options[Inventory.OPTION_EQUIPPED_ITEMS_ONLY] == false)
		{
			var removedItemCount = Math.min(inventoryItem.getItemCountUnequipped(), itemCountToRemove);
			inventoryItem.setItemCountUnequipped(inventoryItem.getItemCountUnequipped() - removedItemCount);
			itemCountToRemove -= removedItemCount;
		}
		
		if ((options[Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY] == false) && (itemCountToRemove > 0))
		{
			var removedItemCount = Math.min(inventoryItem.getItemCountEquipped(), itemCountToRemove);
			inventoryItem.setItemCountEquipped(inventoryItem.getItemCountEquipped() - removedItemCount);
			itemCountToRemove -= removedItemCount;
		}
		
		if (inventoryItem.getItemCountTotal() == 0)
		{
			delete this._inventoryItemLists[inventoryItemTemplateId];
		}
		
		this._parentActor.updateActorStatTargetActorEffectsFromInventoryItem(inventoryItem);
		
		this.updateRegisteredPanelListItem(inventoryItem);
	},
	
	registerPanel : function(guiPanel)
	{
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			if (this._registeredPanelList[i] === guiPanel)
			{
				return;
			}
		}

		this._registeredPanelList.push(guiPanel);
	},

	unequipItem : function(inventoryItemTemplateId, itemCount, options)
	{
		options[Inventory.OPTION_FORCE_EQUIP] = ((options[Inventory.OPTION_FORCE_EQUIP] != null) ? options[Inventory.OPTION_FORCE_EQUIP] : false);

		var inventoryItem = this.getItem(inventoryItemTemplateId);
		if (inventoryItem == null)
		{
			return;
		}

		if (inventoryItem.getItemCountEquipped() == 0)
		{
			return;
		}
		
		var itemCountToChange = Math.min(itemCount, inventoryItem.getItemCountEquipped());
		inventoryItem.setItemCountUnequipped(inventoryItem.getItemCountUnequipped() + itemCountToChange);
		inventoryItem.setItemCountEquipped(inventoryItem.getItemCountEquipped() - itemCountToChange);

		this._parentActor.updateActorStatTargetActorEffectsFromInventoryItem(inventoryItem);

		this.updateRegisteredPanelListItem(inventoryItem);
	},
	
	unregisterPanel : function(guiPanel)
	{
		for (var i = this._registeredPanelList.length - 1; i >= 0; i--)
		{
			if (this._registeredPanelList[i] === guiPanel)
			{
				this._registeredPanelList.splice(i, 1);
			}
		}
	},

	updateRegisteredPanelListItem : function(inventoryItem)
	{
		for (var i = 0; i < this._registeredPanelList.length; i++)
		{
			this._registeredPanelList[i].updateInventoryItem(inventoryItem, true);
		}
	},
	
	//------------------------------------------------------------------------------------------------------------
	//GETTERS - SETTERS
	//------------------------------------------------------------------------------------------------------------

	getItem : function(inventoryItemTemplateId)
	{
		for (var key in this._inventoryItemLists)
		{
			for (var i = 0; i < this._inventoryItemLists[key].length; i++)
			{
				var inventoryItem = this._inventoryItemLists[key][i]; 
				if (inventoryItem.getInventoryItemTemplate().getId() == inventoryItemTemplateId)
				{
					return inventoryItem;
				}
			}
		}
		
		return null;
	},
	
	getItemCount : function(inventoryItemTemplateId, options)
	{
		var inventoryItem = this.getItem(inventoryItemTemplateId);
		if (inventoryItem == null)
		{
			return 0;
		}
		
		if (options[Inventory.OPTION_EQUIPPED_ITEMS_ONLY] == true)
		{
			return inventoryItem.getItemCountEquipped();
		}
		else if (options[Inventory.OPTION_UNEQUIPPED_ITEMS_ONLY] == true)
		{
			return inventoryItem.getItemCountUnequipped();
		}

		return inventoryItem.getItemCountTotal();
	},

	getItemLists : function()
	{
		return this._inventoryItemLists;
	},
	
	getParentActor : function()
	{
		return this._parentActor;
	},
	setParentActor : function(value)
	{
		this._parentActor = value;
	}

	//------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------
//EXPORTS
//------------------------------------------------------------------------------------------------------------

module.exports = Inventory;

//------------------------------------------------------------------------------------------------------------